pub struct AxisConfig { /* private fields */ }Expand description
Axis configuration shared across all series in a plot.
The axis configuration is owned by Plot and affects
all series within the plot. Each series contributes data only; axes control
scaling, ticks, formatting, and grid/border appearance.
Implementations
Sourceimpl AxisConfig
impl AxisConfig
Sourcepub fn new() -> Self
pub fn new() -> Self
Create a new axis configuration.
Use AxisConfig::builder for a fluent configuration style.
Sourcepub fn builder() -> AxisConfigBuilder
pub fn builder() -> AxisConfigBuilder
Start building an axis configuration.
Examples found in repository?
examples/basic.rs (line 38)
7fn main() {
8 Application::new().run(|cx| {
9 let options = WindowOptions {
10 window_bounds: Some(WindowBounds::Windowed(Bounds::centered(
11 None,
12 size(px(720.0), px(480.0)),
13 cx,
14 ))),
15 ..Default::default()
16 };
17
18 cx.open_window(options, |_window, cx| {
19 let series = Series::from_iter_y(
20 "signal",
21 (0..400).map(|i| {
22 let x = i as f64 * 0.03;
23 x.sin()
24 }),
25 SeriesKind::Line(LineStyle {
26 color: Rgba {
27 r: 0.2,
28 g: 0.75,
29 b: 0.95,
30 a: 1.0,
31 },
32 width: 2.0,
33 }),
34 );
35
36 let mut plot = Plot::builder()
37 .theme(Theme::dark())
38 .x_axis(AxisConfig::builder().title("Sample").build())
39 .y_axis(AxisConfig::builder().title("Amplitude").build())
40 .build();
41 plot.add_series(&series);
42
43 let config = PlotViewConfig {
44 show_legend: true,
45 show_hover: true,
46 ..Default::default()
47 };
48
49 let view = PlotView::with_config(plot, config);
50 cx.new(|_| view)
51 })
52 .unwrap();
53 });
54}More examples
examples/advanced.rs (line 104)
34fn build_views(
35 cx: &mut gpui::App,
36) -> (
37 gpui::Entity<PlotView>,
38 gpui::Entity<PlotView>,
39 Series,
40 Series,
41) {
42 let mut stream_a = Series::line("stream-A").with_kind(SeriesKind::Line(LineStyle {
43 color: Rgba {
44 r: 0.2,
45 g: 0.82,
46 b: 0.95,
47 a: 1.0,
48 },
49 width: 2.0,
50 }));
51 let mut stream_b = Series::line("stream-B").with_kind(SeriesKind::Line(LineStyle {
52 color: Rgba {
53 r: 0.95,
54 g: 0.64,
55 b: 0.28,
56 a: 1.0,
57 },
58 width: 2.0,
59 }));
60
61 for i in 0..1_000 {
62 let phase = i as f64 * 0.02;
63 let _ = stream_a.push_y((phase * 0.9).sin() + 0.2 * (phase * 0.13).cos());
64 let _ = stream_b.push_y((phase * 0.45).cos() * 1.15 + 0.15 * (phase * 0.09).sin());
65 }
66
67 let events = Series::from_iter_points(
68 "events(scatter)",
69 (0..200).map(|i| {
70 let x = i as f64 * 80.0 + 40.0;
71 let y = (x * 0.02).sin() * 0.9;
72 gpui_liveplot::Point::new(x, y)
73 }),
74 SeriesKind::Scatter(MarkerStyle {
75 color: Rgba {
76 r: 0.95,
77 g: 0.25,
78 b: 0.55,
79 a: 1.0,
80 },
81 size: 5.0,
82 shape: MarkerShape::Circle,
83 }),
84 );
85
86 let baseline = Series::from_explicit_callback(
87 "baseline(callback)",
88 |x| (x * 0.015).sin() * 0.4,
89 Range::new(0.0, 25_000.0),
90 5_000,
91 SeriesKind::Line(LineStyle {
92 color: Rgba {
93 r: 0.45,
94 g: 0.45,
95 b: 0.5,
96 a: 0.8,
97 },
98 width: 1.0,
99 }),
100 );
101
102 let mut top_plot = Plot::builder()
103 .theme(Theme::dark())
104 .x_axis(AxisConfig::builder().title("Sample").build())
105 .y_axis(AxisConfig::builder().title("Top: stream + events").build())
106 .view(View::FollowLastN { points: 2_000 })
107 .build();
108 top_plot.add_series(&stream_a);
109 top_plot.add_series(&events);
110
111 let mut bottom_plot = Plot::builder()
112 .theme(Theme::dark())
113 .x_axis(AxisConfig::builder().title("Sample").build())
114 .y_axis(
115 AxisConfig::builder()
116 .title("Bottom: stream + baseline")
117 .build(),
118 )
119 .view(View::FollowLastNXY { points: 2_000 })
120 .build();
121 bottom_plot.add_series(&stream_b);
122 bottom_plot.add_series(&baseline);
123
124 let config = PlotViewConfig {
125 show_legend: true,
126 show_hover: true,
127 ..Default::default()
128 };
129
130 let link_group = PlotLinkGroup::new();
131 let options = PlotLinkOptions {
132 link_x: true,
133 link_y: false,
134 link_cursor: true,
135 link_brush: true,
136 link_reset: true,
137 };
138
139 let top = cx.new(|_| {
140 PlotView::with_config(top_plot, config.clone()).with_link_group(link_group.clone(), options)
141 });
142 let bottom =
143 cx.new(|_| PlotView::with_config(bottom_plot, config).with_link_group(link_group, options));
144
145 (top, bottom, stream_a, stream_b)
146}Sourcepub fn formatter(&self) -> &AxisFormatter
pub fn formatter(&self) -> &AxisFormatter
Access the formatter.
Sourcepub fn format_value(&self, value: f64) -> String
pub fn format_value(&self, value: f64) -> String
Format a value for display using the configured formatter.
Sourcepub fn tick_config(&self) -> TickConfig
pub fn tick_config(&self) -> TickConfig
Access the tick configuration.
Sourcepub fn show_minor_grid(&self) -> bool
pub fn show_minor_grid(&self) -> bool
Check if minor grid lines are enabled.
Sourcepub fn show_zero_line(&self) -> bool
pub fn show_zero_line(&self) -> bool
Check if the zero line is enabled.
Sourcepub fn show_border(&self) -> bool
pub fn show_border(&self) -> bool
Check if the axis border is enabled.
Sourcepub fn label_size(&self) -> f32
pub fn label_size(&self) -> f32
Access the tick label font size.