Plot widget entry points and builders.
A Plot owns axis configuration, view mode, and a set of series. All
series in a plot share the same axes and transforms.
Structs
- Plot
- Main plot widget container.
- Plot
Builder - Builder for configuring a plot before construction.