pub struct AxisConfig { /* private fields */ }Expand description
Axis configuration shared across all series in a plot.
The axis configuration is owned by Plot and affects
all series within the plot. Each series contributes data only; axes control
scaling, ticks, formatting, and grid/border appearance.
Implementations
Sourceimpl AxisConfig
impl AxisConfig
Sourcepub fn new() -> Self
pub fn new() -> Self
Create a new axis configuration.
Use AxisConfig::builder for a fluent configuration style.
Sourcepub fn builder() -> AxisConfigBuilder
pub fn builder() -> AxisConfigBuilder
Start building an axis configuration.
Examples found in repository?
examples/basic.rs (line 33)
7fn main() {
8 Application::new().run(|cx| {
9 let options = WindowOptions {
10 window_bounds: Some(WindowBounds::Windowed(Bounds::centered(
11 None,
12 size(px(720.0), px(480.0)),
13 cx,
14 ))),
15 ..Default::default()
16 };
17
18 cx.open_window(options, |_window, cx| {
19 let series = Series::from_iter_y(
20 "signal",
21 (0..400).map(|i| {
22 let x = i as f64 * 0.03;
23 x.sin()
24 }),
25 SeriesKind::Line(LineStyle {
26 color: Color::new(0.2, 0.75, 0.95, 1.0),
27 width: 2.0,
28 }),
29 );
30
31 let mut plot = Plot::builder()
32 .theme(Theme::dark())
33 .x_axis(AxisConfig::builder().title("Sample").build())
34 .y_axis(AxisConfig::builder().title("Amplitude").build())
35 .build();
36 plot.add_series(&series);
37
38 let config = PlotViewConfig {
39 show_legend: true,
40 show_hover: true,
41 ..Default::default()
42 };
43
44 let view = GpuiPlotView::with_config(plot, config);
45 cx.new(|_| view)
46 })
47 .unwrap();
48 });
49}More examples
examples/advanced.rs (line 84)
34fn build_views(
35 cx: &mut gpui::App,
36) -> (
37 gpui::Entity<GpuiPlotView>,
38 gpui::Entity<GpuiPlotView>,
39 Series,
40 Series,
41) {
42 let mut stream_a = Series::line("stream-A").with_kind(SeriesKind::Line(LineStyle {
43 color: Color::new(0.2, 0.82, 0.95, 1.0),
44 width: 2.0,
45 }));
46 let mut stream_b = Series::line("stream-B").with_kind(SeriesKind::Line(LineStyle {
47 color: Color::new(0.95, 0.64, 0.28, 1.0),
48 width: 2.0,
49 }));
50
51 for i in 0..1_000 {
52 let phase = i as f64 * 0.02;
53 let _ = stream_a.push_y((phase * 0.9).sin() + 0.2 * (phase * 0.13).cos());
54 let _ = stream_b.push_y((phase * 0.45).cos() * 1.15 + 0.15 * (phase * 0.09).sin());
55 }
56
57 let events = Series::from_iter_points(
58 "events(scatter)",
59 (0..200).map(|i| {
60 let x = i as f64 * 80.0 + 40.0;
61 let y = (x * 0.02).sin() * 0.9;
62 gpui_liveplot::Point::new(x, y)
63 }),
64 SeriesKind::Scatter(MarkerStyle {
65 color: Color::new(0.95, 0.25, 0.55, 1.0),
66 size: 5.0,
67 shape: MarkerShape::Circle,
68 }),
69 );
70
71 let baseline = Series::from_explicit_callback(
72 "baseline(callback)",
73 |x| (x * 0.015).sin() * 0.4,
74 Range::new(0.0, 25_000.0),
75 5_000,
76 SeriesKind::Line(LineStyle {
77 color: Color::new(0.45, 0.45, 0.5, 0.8),
78 width: 1.0,
79 }),
80 );
81
82 let mut top_plot = Plot::builder()
83 .theme(Theme::dark())
84 .x_axis(AxisConfig::builder().title("Sample").build())
85 .y_axis(AxisConfig::builder().title("Top: stream + events").build())
86 .view(View::FollowLastN { points: 2_000 })
87 .build();
88 top_plot.add_series(&stream_a);
89 top_plot.add_series(&events);
90
91 let mut bottom_plot = Plot::builder()
92 .theme(Theme::dark())
93 .x_axis(AxisConfig::builder().title("Sample").build())
94 .y_axis(
95 AxisConfig::builder()
96 .title("Bottom: stream + baseline")
97 .build(),
98 )
99 .view(View::FollowLastNXY { points: 2_000 })
100 .build();
101 bottom_plot.add_series(&stream_b);
102 bottom_plot.add_series(&baseline);
103
104 let config = PlotViewConfig {
105 show_legend: true,
106 show_hover: true,
107 ..Default::default()
108 };
109
110 let link_group = PlotLinkGroup::new();
111 let options = PlotLinkOptions {
112 link_x: true,
113 link_y: false,
114 link_cursor: true,
115 link_brush: true,
116 link_reset: true,
117 };
118
119 let top = cx.new(|_| {
120 GpuiPlotView::with_config(top_plot, config.clone())
121 .with_link_group(link_group.clone(), options)
122 });
123 let bottom = cx.new(|_| {
124 GpuiPlotView::with_config(bottom_plot, config).with_link_group(link_group, options)
125 });
126
127 (top, bottom, stream_a, stream_b)
128}Sourcepub fn formatter(&self) -> &AxisFormatter
pub fn formatter(&self) -> &AxisFormatter
Access the formatter.
Sourcepub fn format_value(&self, value: f64) -> String
pub fn format_value(&self, value: f64) -> String
Format a value for display using the configured formatter.
Sourcepub fn tick_config(&self) -> TickConfig
pub fn tick_config(&self) -> TickConfig
Access the tick configuration.
Sourcepub fn show_minor_grid(&self) -> bool
pub fn show_minor_grid(&self) -> bool
Check if minor grid lines are enabled.
Sourcepub fn show_zero_line(&self) -> bool
pub fn show_zero_line(&self) -> bool
Check if the zero line is enabled.
Sourcepub fn show_border(&self) -> bool
pub fn show_border(&self) -> bool
Check if the axis border is enabled.
Sourcepub fn label_size(&self) -> f32
pub fn label_size(&self) -> f32
Access the tick label font size.