pub struct Series { /* private fields */ }Expand description
Plot series with data storage and styling.
Series own their data and provide append-only methods for streaming workloads. All axes and transforms are handled at the plot level.
Use Series::share when multiple plots should observe the same live
append-only stream. By contrast, Clone creates an independent copy.
Implementations
Sourceimpl Series
impl Series
Sourcepub fn line(name: impl Into<String>) -> Self
pub fn line(name: impl Into<String>) -> Self
Create a line series with indexed data.
Indexed data uses implicit X values (0, 1, 2, …).
Examples found in repository?
34fn build_views(
35 cx: &mut gpui::App,
36) -> (
37 gpui::Entity<GpuiPlotView>,
38 gpui::Entity<GpuiPlotView>,
39 Series,
40 Series,
41) {
42 let mut stream_a = Series::line("stream-A").with_kind(SeriesKind::Line(LineStyle {
43 color: Color::new(0.2, 0.82, 0.95, 1.0),
44 width: 2.0,
45 }));
46 let mut stream_b = Series::line("stream-B").with_kind(SeriesKind::Line(LineStyle {
47 color: Color::new(0.95, 0.64, 0.28, 1.0),
48 width: 2.0,
49 }));
50
51 for i in 0..1_000 {
52 let phase = i as f64 * 0.02;
53 let _ = stream_a.push_y((phase * 0.9).sin() + 0.2 * (phase * 0.13).cos());
54 let _ = stream_b.push_y((phase * 0.45).cos() * 1.15 + 0.15 * (phase * 0.09).sin());
55 }
56
57 let events = Series::from_iter_points(
58 "events(scatter)",
59 (0..200).map(|i| {
60 let x = i as f64 * 80.0 + 40.0;
61 let y = (x * 0.02).sin() * 0.9;
62 gpui_liveplot::Point::new(x, y)
63 }),
64 SeriesKind::Scatter(MarkerStyle {
65 color: Color::new(0.95, 0.25, 0.55, 1.0),
66 size: 5.0,
67 shape: MarkerShape::Circle,
68 }),
69 );
70
71 let baseline = Series::from_explicit_callback(
72 "baseline(callback)",
73 |x| (x * 0.015).sin() * 0.4,
74 Range::new(0.0, 25_000.0),
75 5_000,
76 SeriesKind::Line(LineStyle {
77 color: Color::new(0.45, 0.45, 0.5, 0.8),
78 width: 1.0,
79 }),
80 );
81
82 let mut top_plot = Plot::builder()
83 .theme(Theme::dark())
84 .x_axis(AxisConfig::builder().title("Sample").build())
85 .y_axis(AxisConfig::builder().title("Top: stream + events").build())
86 .view(View::FollowLastN { points: 2_000 })
87 .build();
88 top_plot.add_series(&stream_a);
89 top_plot.add_series(&events);
90
91 let mut bottom_plot = Plot::builder()
92 .theme(Theme::dark())
93 .x_axis(AxisConfig::builder().title("Sample").build())
94 .y_axis(
95 AxisConfig::builder()
96 .title("Bottom: stream + baseline")
97 .build(),
98 )
99 .view(View::FollowLastNXY { points: 2_000 })
100 .build();
101 bottom_plot.add_series(&stream_b);
102 bottom_plot.add_series(&baseline);
103
104 let config = PlotViewConfig {
105 show_legend: true,
106 show_hover: true,
107 ..Default::default()
108 };
109
110 let link_group = PlotLinkGroup::new();
111 let options = PlotLinkOptions {
112 link_x: true,
113 link_y: false,
114 link_cursor: true,
115 link_brush: true,
116 link_reset: true,
117 };
118
119 let top = cx.new(|_| {
120 GpuiPlotView::with_config(top_plot, config.clone())
121 .with_link_group(link_group.clone(), options)
122 });
123 let bottom = cx.new(|_| {
124 GpuiPlotView::with_config(bottom_plot, config).with_link_group(link_group, options)
125 });
126
127 (top, bottom, stream_a, stream_b)
128}Sourcepub fn scatter(name: impl Into<String>) -> Self
pub fn scatter(name: impl Into<String>) -> Self
Create a scatter series with indexed data.
Indexed data uses implicit X values (0, 1, 2, …).
Sourcepub fn from_iter_y<I, T>(
name: impl Into<String>,
iter: I,
kind: SeriesKind,
) -> Self
pub fn from_iter_y<I, T>( name: impl Into<String>, iter: I, kind: SeriesKind, ) -> Self
Build a series from an iterator of Y values.
X values are assigned as implicit indices.
Examples found in repository?
7fn main() {
8 Application::new().run(|cx| {
9 let options = WindowOptions {
10 window_bounds: Some(WindowBounds::Windowed(Bounds::centered(
11 None,
12 size(px(720.0), px(480.0)),
13 cx,
14 ))),
15 ..Default::default()
16 };
17
18 cx.open_window(options, |_window, cx| {
19 let series = Series::from_iter_y(
20 "signal",
21 (0..400).map(|i| {
22 let x = i as f64 * 0.03;
23 x.sin()
24 }),
25 SeriesKind::Line(LineStyle {
26 color: Color::new(0.2, 0.75, 0.95, 1.0),
27 width: 2.0,
28 }),
29 );
30
31 let mut plot = Plot::builder()
32 .theme(Theme::dark())
33 .x_axis(AxisConfig::builder().title("Sample").build())
34 .y_axis(AxisConfig::builder().title("Amplitude").build())
35 .build();
36 plot.add_series(&series);
37
38 let config = PlotViewConfig {
39 show_legend: true,
40 show_hover: true,
41 ..Default::default()
42 };
43
44 let view = GpuiPlotView::with_config(plot, config);
45 cx.new(|_| view)
46 })
47 .unwrap();
48 });
49}Sourcepub fn from_iter_points<I>(
name: impl Into<String>,
iter: I,
kind: SeriesKind,
) -> Selfwhere
I: IntoIterator<Item = Point>,
pub fn from_iter_points<I>(
name: impl Into<String>,
iter: I,
kind: SeriesKind,
) -> Selfwhere
I: IntoIterator<Item = Point>,
Build a series from an iterator of points.
X values are taken from each Point.
Examples found in repository?
34fn build_views(
35 cx: &mut gpui::App,
36) -> (
37 gpui::Entity<GpuiPlotView>,
38 gpui::Entity<GpuiPlotView>,
39 Series,
40 Series,
41) {
42 let mut stream_a = Series::line("stream-A").with_kind(SeriesKind::Line(LineStyle {
43 color: Color::new(0.2, 0.82, 0.95, 1.0),
44 width: 2.0,
45 }));
46 let mut stream_b = Series::line("stream-B").with_kind(SeriesKind::Line(LineStyle {
47 color: Color::new(0.95, 0.64, 0.28, 1.0),
48 width: 2.0,
49 }));
50
51 for i in 0..1_000 {
52 let phase = i as f64 * 0.02;
53 let _ = stream_a.push_y((phase * 0.9).sin() + 0.2 * (phase * 0.13).cos());
54 let _ = stream_b.push_y((phase * 0.45).cos() * 1.15 + 0.15 * (phase * 0.09).sin());
55 }
56
57 let events = Series::from_iter_points(
58 "events(scatter)",
59 (0..200).map(|i| {
60 let x = i as f64 * 80.0 + 40.0;
61 let y = (x * 0.02).sin() * 0.9;
62 gpui_liveplot::Point::new(x, y)
63 }),
64 SeriesKind::Scatter(MarkerStyle {
65 color: Color::new(0.95, 0.25, 0.55, 1.0),
66 size: 5.0,
67 shape: MarkerShape::Circle,
68 }),
69 );
70
71 let baseline = Series::from_explicit_callback(
72 "baseline(callback)",
73 |x| (x * 0.015).sin() * 0.4,
74 Range::new(0.0, 25_000.0),
75 5_000,
76 SeriesKind::Line(LineStyle {
77 color: Color::new(0.45, 0.45, 0.5, 0.8),
78 width: 1.0,
79 }),
80 );
81
82 let mut top_plot = Plot::builder()
83 .theme(Theme::dark())
84 .x_axis(AxisConfig::builder().title("Sample").build())
85 .y_axis(AxisConfig::builder().title("Top: stream + events").build())
86 .view(View::FollowLastN { points: 2_000 })
87 .build();
88 top_plot.add_series(&stream_a);
89 top_plot.add_series(&events);
90
91 let mut bottom_plot = Plot::builder()
92 .theme(Theme::dark())
93 .x_axis(AxisConfig::builder().title("Sample").build())
94 .y_axis(
95 AxisConfig::builder()
96 .title("Bottom: stream + baseline")
97 .build(),
98 )
99 .view(View::FollowLastNXY { points: 2_000 })
100 .build();
101 bottom_plot.add_series(&stream_b);
102 bottom_plot.add_series(&baseline);
103
104 let config = PlotViewConfig {
105 show_legend: true,
106 show_hover: true,
107 ..Default::default()
108 };
109
110 let link_group = PlotLinkGroup::new();
111 let options = PlotLinkOptions {
112 link_x: true,
113 link_y: false,
114 link_cursor: true,
115 link_brush: true,
116 link_reset: true,
117 };
118
119 let top = cx.new(|_| {
120 GpuiPlotView::with_config(top_plot, config.clone())
121 .with_link_group(link_group.clone(), options)
122 });
123 let bottom = cx.new(|_| {
124 GpuiPlotView::with_config(bottom_plot, config).with_link_group(link_group, options)
125 });
126
127 (top, bottom, stream_a, stream_b)
128}Sourcepub fn from_explicit_callback(
name: impl Into<String>,
function: impl Fn(f64) -> f64,
x_range: Range,
points: usize,
kind: SeriesKind,
) -> Self
pub fn from_explicit_callback( name: impl Into<String>, function: impl Fn(f64) -> f64, x_range: Range, points: usize, kind: SeriesKind, ) -> Self
Build a series by sampling a callback function.
The callback is sampled uniformly across x_range.
Examples found in repository?
34fn build_views(
35 cx: &mut gpui::App,
36) -> (
37 gpui::Entity<GpuiPlotView>,
38 gpui::Entity<GpuiPlotView>,
39 Series,
40 Series,
41) {
42 let mut stream_a = Series::line("stream-A").with_kind(SeriesKind::Line(LineStyle {
43 color: Color::new(0.2, 0.82, 0.95, 1.0),
44 width: 2.0,
45 }));
46 let mut stream_b = Series::line("stream-B").with_kind(SeriesKind::Line(LineStyle {
47 color: Color::new(0.95, 0.64, 0.28, 1.0),
48 width: 2.0,
49 }));
50
51 for i in 0..1_000 {
52 let phase = i as f64 * 0.02;
53 let _ = stream_a.push_y((phase * 0.9).sin() + 0.2 * (phase * 0.13).cos());
54 let _ = stream_b.push_y((phase * 0.45).cos() * 1.15 + 0.15 * (phase * 0.09).sin());
55 }
56
57 let events = Series::from_iter_points(
58 "events(scatter)",
59 (0..200).map(|i| {
60 let x = i as f64 * 80.0 + 40.0;
61 let y = (x * 0.02).sin() * 0.9;
62 gpui_liveplot::Point::new(x, y)
63 }),
64 SeriesKind::Scatter(MarkerStyle {
65 color: Color::new(0.95, 0.25, 0.55, 1.0),
66 size: 5.0,
67 shape: MarkerShape::Circle,
68 }),
69 );
70
71 let baseline = Series::from_explicit_callback(
72 "baseline(callback)",
73 |x| (x * 0.015).sin() * 0.4,
74 Range::new(0.0, 25_000.0),
75 5_000,
76 SeriesKind::Line(LineStyle {
77 color: Color::new(0.45, 0.45, 0.5, 0.8),
78 width: 1.0,
79 }),
80 );
81
82 let mut top_plot = Plot::builder()
83 .theme(Theme::dark())
84 .x_axis(AxisConfig::builder().title("Sample").build())
85 .y_axis(AxisConfig::builder().title("Top: stream + events").build())
86 .view(View::FollowLastN { points: 2_000 })
87 .build();
88 top_plot.add_series(&stream_a);
89 top_plot.add_series(&events);
90
91 let mut bottom_plot = Plot::builder()
92 .theme(Theme::dark())
93 .x_axis(AxisConfig::builder().title("Sample").build())
94 .y_axis(
95 AxisConfig::builder()
96 .title("Bottom: stream + baseline")
97 .build(),
98 )
99 .view(View::FollowLastNXY { points: 2_000 })
100 .build();
101 bottom_plot.add_series(&stream_b);
102 bottom_plot.add_series(&baseline);
103
104 let config = PlotViewConfig {
105 show_legend: true,
106 show_hover: true,
107 ..Default::default()
108 };
109
110 let link_group = PlotLinkGroup::new();
111 let options = PlotLinkOptions {
112 link_x: true,
113 link_y: false,
114 link_cursor: true,
115 link_brush: true,
116 link_reset: true,
117 };
118
119 let top = cx.new(|_| {
120 GpuiPlotView::with_config(top_plot, config.clone())
121 .with_link_group(link_group.clone(), options)
122 });
123 let bottom = cx.new(|_| {
124 GpuiPlotView::with_config(bottom_plot, config).with_link_group(link_group, options)
125 });
126
127 (top, bottom, stream_a, stream_b)
128}Sourcepub fn kind(&self) -> &SeriesKind
pub fn kind(&self) -> &SeriesKind
Access the series kind.
Sourcepub fn with_kind(self, kind: SeriesKind) -> Self
pub fn with_kind(self, kind: SeriesKind) -> Self
Replace the series kind.
Examples found in repository?
34fn build_views(
35 cx: &mut gpui::App,
36) -> (
37 gpui::Entity<GpuiPlotView>,
38 gpui::Entity<GpuiPlotView>,
39 Series,
40 Series,
41) {
42 let mut stream_a = Series::line("stream-A").with_kind(SeriesKind::Line(LineStyle {
43 color: Color::new(0.2, 0.82, 0.95, 1.0),
44 width: 2.0,
45 }));
46 let mut stream_b = Series::line("stream-B").with_kind(SeriesKind::Line(LineStyle {
47 color: Color::new(0.95, 0.64, 0.28, 1.0),
48 width: 2.0,
49 }));
50
51 for i in 0..1_000 {
52 let phase = i as f64 * 0.02;
53 let _ = stream_a.push_y((phase * 0.9).sin() + 0.2 * (phase * 0.13).cos());
54 let _ = stream_b.push_y((phase * 0.45).cos() * 1.15 + 0.15 * (phase * 0.09).sin());
55 }
56
57 let events = Series::from_iter_points(
58 "events(scatter)",
59 (0..200).map(|i| {
60 let x = i as f64 * 80.0 + 40.0;
61 let y = (x * 0.02).sin() * 0.9;
62 gpui_liveplot::Point::new(x, y)
63 }),
64 SeriesKind::Scatter(MarkerStyle {
65 color: Color::new(0.95, 0.25, 0.55, 1.0),
66 size: 5.0,
67 shape: MarkerShape::Circle,
68 }),
69 );
70
71 let baseline = Series::from_explicit_callback(
72 "baseline(callback)",
73 |x| (x * 0.015).sin() * 0.4,
74 Range::new(0.0, 25_000.0),
75 5_000,
76 SeriesKind::Line(LineStyle {
77 color: Color::new(0.45, 0.45, 0.5, 0.8),
78 width: 1.0,
79 }),
80 );
81
82 let mut top_plot = Plot::builder()
83 .theme(Theme::dark())
84 .x_axis(AxisConfig::builder().title("Sample").build())
85 .y_axis(AxisConfig::builder().title("Top: stream + events").build())
86 .view(View::FollowLastN { points: 2_000 })
87 .build();
88 top_plot.add_series(&stream_a);
89 top_plot.add_series(&events);
90
91 let mut bottom_plot = Plot::builder()
92 .theme(Theme::dark())
93 .x_axis(AxisConfig::builder().title("Sample").build())
94 .y_axis(
95 AxisConfig::builder()
96 .title("Bottom: stream + baseline")
97 .build(),
98 )
99 .view(View::FollowLastNXY { points: 2_000 })
100 .build();
101 bottom_plot.add_series(&stream_b);
102 bottom_plot.add_series(&baseline);
103
104 let config = PlotViewConfig {
105 show_legend: true,
106 show_hover: true,
107 ..Default::default()
108 };
109
110 let link_group = PlotLinkGroup::new();
111 let options = PlotLinkOptions {
112 link_x: true,
113 link_y: false,
114 link_cursor: true,
115 link_brush: true,
116 link_reset: true,
117 };
118
119 let top = cx.new(|_| {
120 GpuiPlotView::with_config(top_plot, config.clone())
121 .with_link_group(link_group.clone(), options)
122 });
123 let bottom = cx.new(|_| {
124 GpuiPlotView::with_config(bottom_plot, config).with_link_group(link_group, options)
125 });
126
127 (top, bottom, stream_a, stream_b)
128}Create another series handle that shares the same append-only data.
The returned series receives a new SeriesId, so it can coexist with
the source series in the same plot. Data appends through either series
are immediately visible to all shared handles.
Sourcepub fn push_y(&mut self, y: f64) -> Result<usize, AppendError>
pub fn push_y(&mut self, y: f64) -> Result<usize, AppendError>
Append a Y value to an indexed series.
Examples found in repository?
34fn build_views(
35 cx: &mut gpui::App,
36) -> (
37 gpui::Entity<GpuiPlotView>,
38 gpui::Entity<GpuiPlotView>,
39 Series,
40 Series,
41) {
42 let mut stream_a = Series::line("stream-A").with_kind(SeriesKind::Line(LineStyle {
43 color: Color::new(0.2, 0.82, 0.95, 1.0),
44 width: 2.0,
45 }));
46 let mut stream_b = Series::line("stream-B").with_kind(SeriesKind::Line(LineStyle {
47 color: Color::new(0.95, 0.64, 0.28, 1.0),
48 width: 2.0,
49 }));
50
51 for i in 0..1_000 {
52 let phase = i as f64 * 0.02;
53 let _ = stream_a.push_y((phase * 0.9).sin() + 0.2 * (phase * 0.13).cos());
54 let _ = stream_b.push_y((phase * 0.45).cos() * 1.15 + 0.15 * (phase * 0.09).sin());
55 }
56
57 let events = Series::from_iter_points(
58 "events(scatter)",
59 (0..200).map(|i| {
60 let x = i as f64 * 80.0 + 40.0;
61 let y = (x * 0.02).sin() * 0.9;
62 gpui_liveplot::Point::new(x, y)
63 }),
64 SeriesKind::Scatter(MarkerStyle {
65 color: Color::new(0.95, 0.25, 0.55, 1.0),
66 size: 5.0,
67 shape: MarkerShape::Circle,
68 }),
69 );
70
71 let baseline = Series::from_explicit_callback(
72 "baseline(callback)",
73 |x| (x * 0.015).sin() * 0.4,
74 Range::new(0.0, 25_000.0),
75 5_000,
76 SeriesKind::Line(LineStyle {
77 color: Color::new(0.45, 0.45, 0.5, 0.8),
78 width: 1.0,
79 }),
80 );
81
82 let mut top_plot = Plot::builder()
83 .theme(Theme::dark())
84 .x_axis(AxisConfig::builder().title("Sample").build())
85 .y_axis(AxisConfig::builder().title("Top: stream + events").build())
86 .view(View::FollowLastN { points: 2_000 })
87 .build();
88 top_plot.add_series(&stream_a);
89 top_plot.add_series(&events);
90
91 let mut bottom_plot = Plot::builder()
92 .theme(Theme::dark())
93 .x_axis(AxisConfig::builder().title("Sample").build())
94 .y_axis(
95 AxisConfig::builder()
96 .title("Bottom: stream + baseline")
97 .build(),
98 )
99 .view(View::FollowLastNXY { points: 2_000 })
100 .build();
101 bottom_plot.add_series(&stream_b);
102 bottom_plot.add_series(&baseline);
103
104 let config = PlotViewConfig {
105 show_legend: true,
106 show_hover: true,
107 ..Default::default()
108 };
109
110 let link_group = PlotLinkGroup::new();
111 let options = PlotLinkOptions {
112 link_x: true,
113 link_y: false,
114 link_cursor: true,
115 link_brush: true,
116 link_reset: true,
117 };
118
119 let top = cx.new(|_| {
120 GpuiPlotView::with_config(top_plot, config.clone())
121 .with_link_group(link_group.clone(), options)
122 });
123 let bottom = cx.new(|_| {
124 GpuiPlotView::with_config(bottom_plot, config).with_link_group(link_group, options)
125 });
126
127 (top, bottom, stream_a, stream_b)
128}Sourcepub fn extend_y<I, T>(&mut self, values: I) -> Result<usize, AppendError>
pub fn extend_y<I, T>(&mut self, values: I) -> Result<usize, AppendError>
Append multiple Y values to an indexed series.
Returns the number of appended points.
Examples found in repository?
130fn spawn_updates(
131 window: &mut gpui::Window,
132 cx: &mut gpui::App,
133 top: gpui::Entity<GpuiPlotView>,
134 bottom: gpui::Entity<GpuiPlotView>,
135 mut stream_a: Series,
136 mut stream_b: Series,
137) {
138 window
139 .spawn(cx, move |cx: &mut AsyncWindowContext| {
140 let mut cx = cx.clone();
141 async move {
142 let mut phase = 0.0_f64;
143 loop {
144 Timer::after(Duration::from_millis(16)).await;
145 let _ = stream_a.extend_y((0..120).map(|_| {
146 let y = (phase * 0.9).sin() + 0.2 * (phase * 0.13).cos();
147 phase += 0.02;
148 y
149 }));
150 let _ = stream_b.extend_y((0..120).map(|_| {
151 let y = (phase * 0.45).cos() * 1.15 + 0.15 * (phase * 0.09).sin();
152 phase += 0.02;
153 y
154 }));
155
156 let _ = cx.update(|_, cx| {
157 top.update(cx, |_view, view_cx| view_cx.notify());
158 bottom.update(cx, |_view, view_cx| view_cx.notify());
159 });
160 }
161 }
162 })
163 .detach();
164}Sourcepub fn push_point(&mut self, point: Point) -> Result<usize, AppendError>
pub fn push_point(&mut self, point: Point) -> Result<usize, AppendError>
Append a point to an explicit series.
Sourcepub fn extend_points<I>(&mut self, points: I) -> Result<usize, AppendError>where
I: IntoIterator<Item = Point>,
pub fn extend_points<I>(&mut self, points: I) -> Result<usize, AppendError>where
I: IntoIterator<Item = Point>,
Append multiple explicit points to a series.
Returns the number of appended points when X values stay monotonic.
If any new point has a smaller X than the previous point, all points are
still appended and AppendError::NonMonotonicX is returned.
Sourcepub fn generation(&self) -> u64
pub fn generation(&self) -> u64
Access the series generation.
This monotonically increasing value is used for render cache invalidation.
Sourcepub fn is_visible(&self) -> bool
pub fn is_visible(&self) -> bool
Check if the series is visible.
Sourcepub fn set_visible(&mut self, visible: bool)
pub fn set_visible(&mut self, visible: bool)
Toggle series visibility.